﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Castle.MicroKernel.Registration;
using System.ComponentModel.Composition;
using Castle.Windsor;
using Castle.MicroKernel.SubSystems.Configuration;
using Castle.Core;
using Topics.Radical;
using Topics.Radical.Reflection;
using Castle.Facilities.Startable;
using Topics.Radical.ComponentModel.Messaging;
using Topics.Radical.Messaging;
using Topics.Radical.ComponentModel;
using Topics.Radical.Threading;
using Topics.Radical.ComponentModel.Validation;
using Raven.Client;
using System.Net;
using Raven.Client.Document;

namespace Bootstrapper.Installers
{
	[Export( typeof( IWindsorInstaller ) )]
	public sealed class DefaultInstaller : IWindsorInstaller
	{
		public void Install( IWindsorContainer container, IConfigurationStore store )
		{
			var boot = container.Resolve<Bootstrapper>();
			var directory = boot.GetBootDirectory();

			container.Register
			(
				Component.For<IWindsorContainer>()
					.Instance( container )
			);

			container.Register
			(
				Component.For<IServiceProvider>()
					.Instance( new ServiceProviderWrapper( container ) )
			);

			container.Register
			(
				Component.For<IDocumentStore>()
					.UsingFactoryMethod( ()=>
					{
						var ds = new DocumentStore()
						{
							Url = "http://localhost:8080/",
							DefaultDatabase = "CQRS-ES"
						}.Initialize();

						return ds;
					})
			);

			container.Register
			(
				Types.FromAssemblyInDirectory( new AssemblyFilter( directory, "WebApi*.dll" ) )
					.IncludeNonPublicTypes()
					.Where( t => !t.IsInterface && !t.IsAbstract && t.Is<IHandleMessage>() )
					.WithService.Select( ( type, all ) =>
					{
						var contracts = type.GetInterfaces()
							.Where( i => i.IsAttributeDefined<ContractAttribute>() )
							.ToArray();
						return contracts;
					} )
			);

			container.Register
			(
				Types.FromAssemblyInDirectory( new AssemblyFilter( directory, "WebApi*.dll" ) )
					.IncludeNonPublicTypes()
					.Where( t => !t.IsInterface && !t.IsAbstract && !String.IsNullOrWhiteSpace( t.Namespace ) && t.Namespace.EndsWith( ".Services" ) )
					.WithService.FirstInterface()
					.Configure( c =>
					{
						if ( c.Implementation.Is<IRequireToStart>() )
						{
							c.Forward<IRequireToStart>();
						}
					} )
			);

			container.Register
			(
				Types.FromAssemblyInDirectory( new AssemblyFilter( directory, "WebApi*.dll" ) )
					.Where( t => !t.IsInterface && !t.IsAbstract && t.IsNested && t.Name.EndsWith( "Factory" ) )
					.WithService.Self()
					.LifestyleSingleton()
			);

			container.Register
			(
				Types.FromAssemblyInDirectory( new AssemblyFilter( directory, "WebApi*.dll" ) )
					.Where( t => !t.IsInterface && !t.IsAbstract && !String.IsNullOrWhiteSpace( t.Namespace ) && t.Namespace.EndsWith( ".Validators" ) && t.Is( typeof( IValidator<> ) ) )
					.WithService.Select( ( type, all ) =>
					{
						var contracts = type.GetInterfaces()
							.Where( i => i.IsAttributeDefined<ContractAttribute>() )
							.ToArray();
						return contracts;
					} )
					.LifestyleSingleton()
			);
		}
	}
}
