﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Topics.Radical.Validation;

namespace WebApi.Data.Commits
{
	public class Commit
	{
		public class Factory
		{
			public Commit CreateFor( Guid transactionIdentifier, string correlationId, IAggregate aggregate, String userAccount )
			{
				var commitId = "commits/" + Guid.NewGuid().ToString();
				var commit = new Commit( commitId, transactionIdentifier, correlationId, aggregate.Id, aggregate.GetUncommittedEvents(), userAccount );

				return commit;
			}
		}

		[Raven.Imports.Newtonsoft.Json.JsonConstructor]
		private Commit()
		{

		}

		private Commit( String commitId, Guid transactionIdentifier, String correlationId, String aggregateId, IEnumerable<IDomainEvent> uncommittedEvents, String userAccount )
		{
			this.Id = commitId;
			this.CreatedOn = DateTimeOffset.Now;
			this.TransactionIdentifier = transactionIdentifier.ToString();
			this.CorrelationId = correlationId;
			this.AggregateIdentifier = aggregateId;
			this.Events = uncommittedEvents;
			this.UserAccount = userAccount;
		}

		public String Id { get; private set; }
		public DateTimeOffset CreatedOn { get; private set; }
		public String TransactionIdentifier { get; private set; }
		public String CorrelationId { get; private set; }
		public String AggregateIdentifier { get; private set; }
		public IEnumerable<IDomainEvent> Events { get; private set; }

		public Boolean IsDispatched { get; private set; }

		public void MarkAsDispatched()
		{
			this.IsDispatched = true;
		}

		public String UserAccount { get; private set; }
	}
}
