﻿define(['jason/client', 'services/commandsTracking', 'durandal/http', 'durandal/system'],
    function (jason, tracking, http, system) {
        var welcome = function () {
            var self = this;

            self.displayName = 'Implementing CQRS home page';

            self.tracking = tracking;
            self.jason = jason;
            self.http = http;

            self.query = ko.observable('');
            self.suggestions = ko.observableArray();
            self.results = ko.observableArray();

            self.commands = ko.computed(function () {
                return self.tracking.commands();
            });

            self.firstName = ko.observable('Mauro');
            self.lastName = ko.observable('Servienti');
            self.companyName = ko.observable('topics.it');
        };

        welcome.prototype.createNewPerson = function () {
            var self = this;

            self.jason.execute({
                command: {
                    '$type': 'WebApi.Commands.CreateNewPersonCommand, WebApi.Commands',
                    firstName: self.firstName(),
                    lastName: self.lastName()
                },
                commandTracking: {
                    enabled: true,
                    removeOnEvent: true,
                    removeOnFailure: false,
                    eventTypePattern: '*PersonCreated*',
                    onEventReceived: function () {
                        system.log('create person completed.');
                        self.isLoading(false);
                    },
                    description: 'Adding new person "' + self.firstName() + ' ' + self.lastName() + '".'
                }
            });
        };

        welcome.prototype.createNewCompany = function () {
            var self = this;

            self.jason.execute({
                command: {
                    '$type': 'WebApi.Commands.CreateNewCompanyCommand, WebApi.Commands',
                    companyName: self.companyName()
                },
                commandTracking: {
                    enabled: true,
                    removeOnEvent: true,
                    removeOnFailure: false,
                    eventTypePattern: '*CompanyCreated*',
                    onEventReceived: function () {
                        system.log('create company completed.');
                    },
                    description: 'Adding new company "' + self.companyName() + '".'
                }
            });
        };

        welcome.prototype.search = function () {
            var self = this;

            self.suggestions.removeAll();
            self.results.removeAll();

            var q = self.query();

            system.log('search: ', q);

            http.get('/api/search?q=' + q)
                        .then(function (r) {
                            for (i = 0; i < r.suggestions.length; i++) {
                                self.suggestions.push(r.suggestions[i]);
                            }

                            for (i = 0; i < r.results.length; i++) {
                                self.results.push(r.results[i]);
                            }
                        });
        };

        welcome.prototype.searchBySuggestion = function (s) {
            var self = this;

            system.log('searchBySuggestion: ', s);

            self.query(s);
            self.search();
        };

        return welcome;
    });