﻿(function (module) {

    var itemTemplate = function ($compile, frontendBaseUrl, $templateCache, $log, $rootScope, $http) {

        $log.debug('searchResult directive ctor');

        var getDefaultTemplateUrl = function (dataType, templatesBaseUrl) {
            var template = templatesBaseUrl + dataType + '.html';
            return template;
        };

        var trimStartSlash = function (value) {
            var segment = value;
            if (segment.substring(0, 1) === '/') {
                $log.debug('removing the / at the beginning.');
                segment = segment.substring(1);
            }

            return segment;
        };

        var getTemplate = function (dataType, templateMap, templatesBaseUrl) {

            $log.debug('Supplied dataType: ', dataType);
            $log.debug('Supplied templateMap: ', templateMap);
            $log.debug('templatesBaseUrl: ', templatesBaseUrl);

            var templateUrl;
            if (templateMap == undefined) {
                $log.debug('No supplied template map, setting to default value');
                templateUrl = getDefaultTemplateUrl(dataType, templatesBaseUrl);
            }
            else {
                var templateUrl = templateMap[dataType];
                if (templateUrl === '' || templateUrl == null || templateUrl == undefined) {
                    $log.debug('cannot find a template url in the supplied template map, setting to default value');
                    templateUrl = getDefaultTemplateUrl(dataType, templatesBaseUrl);
                }
            }

            if (templateUrl.substring(0, 1) === '^') {
                $log.debug('template url starts with ^, rebasing on the default templates folder.');
                var segment = templateUrl.substring(1);
                segment = trimStartSlash(segment);

                templateUrl = templatesBaseUrl + segment;
            }

            $log.debug('templateUrl is: ', templateUrl);

            var promise = $http.get(templateUrl, { cache: $templateCache });

            return promise;
        }

        return {
            restrict: 'A',
            scope: {
                item: '=',
                templates: '&',
                settings: '&',
            },
            transclude: true,
            replace: true,
            controller: ['$scope', function ($scope) {

            }],
            link: function (scope, element, attrs, controller) {

                var settings = scope.settings();
                if (!settings) {
                    settings = {};
                }

                if (!settings.dataTypeProperty) {
                    settings.dataTypeProperty = 'dataType';
                }

                var rootFolder = frontendBaseUrl + 'templates/';
                if (settings.defaultTemplatesRootFolder) {
                    if (settings.defaultTemplatesRootFolder.substring(0, 1) === '^') {
                        var segment = settings.defaultTemplatesRootFolder.substring(1);
                        segment = trimStartSlash(segment);

                        rootFolder = rootFolder + segment;
                    }
                    else {
                        rootFolder = settings.defaultTemplatesRootFolder
                    }
                }

                var loader = getTemplate(scope.item[settings.dataTypeProperty], scope.templates(), rootFolder);

                var promise = loader.success(function (html) {
                    element.html(html);
                }).then(function (response) {
                    element.replaceWith($compile(element.html())(scope));
                });
            }
        };
    };
    itemTemplate.$inject = ['$compile', 'frontendBaseUrl', '$templateCache', '$log', '$rootScope', '$http'];

    module.directive('itemTemplate', itemTemplate);

}(angular.module('my.directives')));