﻿(function (module) {

    var typeahead = function (frontendBaseUrl, $timeout, $log, $rootScope) {

        $log.debug('typeahead directive ctor.');

        return {
            restrict: 'EA',
            transclude: true,
            replace: true,
            templateUrl: frontendBaseUrl + 'directives/typeaheadDirective.html',
            scope: {
                search: '&',
                select: '&',
                items: '=',
                term: '=',
                placeholder: '='
            },
            controller: ['$scope', function ($scope) {

                $scope.items = [];

                $scope.hide = false;

                this.activate = function (item) {
                    $log.debug('activate:', item);
                    $scope.active = item;
                };

                this.activateNextItem = function () {
                    var index = $scope.items.indexOf($scope.active);

                    this.activate($scope.items[(index + 1) % $scope.items.length]);
                };

                this.activatePreviousItem = function () {
                    var index = $scope.items.indexOf($scope.active);

                    this.activate($scope.items[index === 0 ? $scope.items.length - 1 : index - 1]);
                };

                this.isActive = function (item) {
                    return $scope.active === item;
                };

                this.selectActive = function () {
                    this.select($scope.active);
                };

                this.select = function (item) {

                    $scope.hide = true;
                    $scope.focused = true;

                    var obj = {
                        item: item,
                        refineSearch: function (term) {
                            $scope.term = term;
                            searchHandler();

                            $rootScope.safeApply($scope, function () {
                                $scope.focused = true;
                            });
                        }
                    };

                    $scope.select(obj);
                };

                $scope.isVisible = function () {
                    return !$scope.hide && ($scope.focused || $scope.mousedOver) && $scope.items.length > 0;
                };

                var debouncer;

                var searchHandler = function () {
                    debouncer = null;

                    $rootScope.safeApply($scope, function () {
                        $log.debug('debounce completed -> executing search: [term]', $scope.term);
                        $scope.hide = false;
                        $scope.search({ term: $scope.term });
                    });
                };

                var explicitSearchHandler = function () {
                    searchHandler();

                    $rootScope.safeApply($scope, function () {
                        $scope.focused = true;
                    });
                };

                $scope.query = function () {

                    $log.debug('typehead -> query(): ', $scope.term);

                    if (!debouncer) {
                        debouncer = $.debounce(500, false, searchHandler);
                        debouncer();
                        $log.debug('typehead -> query() -> debouncer started.');
                    }
                }

                $scope.explicitQuery = function () {

                    $log.debug('typehead -> explicitSearchHandler(): ', $scope.term);

                    if (!debouncer) {
                        debouncer = $.debounce(750, false, explicitSearchHandler);
                        debouncer();
                        $log.debug('typehead -> explicitSearchHandler() -> debouncer started.');
                    }
                }
            }],
            link: function (scope, element, attrs, controller) {

                var $input = element.find(' > input');
                var $list = element.find('> div');

                $log.debug('$input:', $input);

                $input.bind('focus', function () {
                    scope.$apply(function () { scope.focused = true; });
                });

                $input.bind('blur', function () {
                    scope.$apply(function () { scope.focused = false; });
                });

                $list.bind('mouseover', function () {
                    scope.$apply(function () { scope.mousedOver = true; });
                });

                $list.bind('mouseleave', function () {
                    scope.$apply(function () { scope.mousedOver = false; });
                });

                $input.bind('keyup', function (e) {
                    if (e.keyCode === 9 || e.keyCode === 13) {
                        scope.$apply(function () { controller.selectActive(); });
                    }

                    if (e.keyCode === 27) {
                        scope.$apply(function () { scope.hide = true; });
                    }
                });

                $input.bind('keydown', function (e) {

                    if (e.keyCode === 9 || e.keyCode === 13 || e.keyCode === 27) {
                        e.preventDefault();
                    };

                    if (e.keyCode === 40) {
                        e.preventDefault();
                        scope.$apply(function () { controller.activateNextItem(); });
                    }

                    if (e.keyCode === 38) {
                        e.preventDefault();
                        scope.$apply(function () { controller.activatePreviousItem(); });
                    }
                });

                scope.$watch('items', function (items) {
                    controller.activate(items.length ? items[0] : null);
                });

                scope.$watch('focused', function (focused) {

                    if (focused) {
                        $timeout(function (){
                            $input.focus();
                            if (!scope.items || scope.items.length == 0) {
                                scope.query();
                            }
                        }, 0, false);
                    }
                });

                scope.$watch('isVisible()', function (visible) {
                    if (visible) {
                        var pos = $input.position();
                        var height = $input[0].offsetHeight;

                        $list.css({
                            top: pos.top + height,
                            left: pos.left,
                            position: 'absolute',
                            display: 'block',
                            'z-index': '9999'
                        });
                    } else {
                        $list.css('display', 'none');
                    }
                });
            }
        };
    };
    typeahead.$inject = ['frontendBaseUrl', '$timeout', '$log', '$rootScope'];

    module.directive('typeahead', typeahead);

}(angular.module('my.directives')));
