﻿(function (module) {
    var jasonClient = function ($log, $http, $q, utilities, tracking, clientNotifications) {

        $log.debug('jasonClient factory ctor.');

        return {
            execute: function (context) {

                context.beforeExecute = context.beforeExecute || function () { };
                context.afterExecute = context.afterExecute || function () { };
                context.executionFailed = context.executionFailed || function () { };
                context.commandTracking = context.commandTracking || { enabled: false };
                if (context.commandTracking.enabled) {
                    context.commandTracking.removeOnFailure = context.commandTracking.removeOnFailure || true;
                    context.commandTracking.removeOnEvent = context.commandTracking.removeOnEvent || true;
                }

                $log.debug('[jasonClient] ready to post command:', context.command);

                var correlationId = utilities.generateIdentifier();

                $log.debug('[jasonClient] correlationId:', correlationId);

                $log.debug('[jasonClient] calling beforeExecute...');
                context.beforeExecute(context.command, correlationId);
                $log.debug('[jasonClient] beforeExecute called:', correlationId);

                if (context.commandTracking.enabled) {

                    $log.debug('adding to commands client notifications via SignalR for:', correlationId);
                    clientNotifications.notifyOnEventByPattern(correlationId, context.commandTracking.eventTypePattern);

                    $log.debug('adding to commands tracking for:', correlationId);
                    tracking.track({
                        correlationId: correlationId,
                        eventTypePattern: context.commandTracking.eventTypePattern || '',
                        onEventReceived: context.commandTracking.onEventReceived || function () { },
                        removeOnFailure: context.commandTracking.removeOnFailure,
                        removeOnEvent: context.commandTracking.removeOnEvent,
                        description: context.commandTracking.description || 'Executing commdand: [' + correlationId + '].'
                    });
                }

                var deferred = $q.defer();

                $http({
                    method: 'POST',
                    url: '/api/jason/post',
                    data: JSON.stringify(context.command),
                    headers: {
                        'Content-Type': 'application/json',
                        'x-correlation-id': correlationId
                    }
                })
                .success(function (data, status, headers, config) {
                    /*
                    - la validazione del comando è andata a buon fine;
                    - il comando dal front-end è passato sul bus senza problemi;
                    */
                    $log.debug('[jasonClient] jason post completed, result:', data, status, headers, config);
                    deferred.resolve(data);
                })
                .error(function (error, status, headers, config) {
                    $log.debug('[jasonClient] jason post failed, result:', error, status, headers, config);
                    if (!utilities.isSuccessStatus(status)) {
                        /*
                        questo fallimento può essere dovuto a:
                            - la validazione del comando fallisce;
                            - è fallita la gestione comando sul front-end;
                        */
                        if (context.commandTracking.enabled && context.commandTracking.removeOnFailure) {
                            tracking.stopTracking({ correlationId: correlationId });
                        }

                        context.executionFailed(error, status);

                        deferred.reject(error);
                    }
                });

                $log.debug('[jasonClient] jason controller invoked.');
                context.afterExecute(context.command, correlationId);

                return deferred.promise;
            }
        };
    };
    jasonClient.$inject = ['$log', '$http', '$q', 'utilities', 'commandsTracking', 'clientNotifications'];

    module.factory("jasonClient", jasonClient);

}(angular.module('my.services')));