﻿using System;
using System.Linq;
using System.Web.Http;
using Raven.Client;
using Raven.Client.Linq;
using WebApi.Data.Indexes;
using WebApiServer.Models;

namespace WebApiServer.Controllers
{
	public class HistoryController : ApiController
	{
		readonly IDocumentStore store;

		public HistoryController( IDocumentStore documentStore )
		{
			this.store = documentStore;
		}

		public PagedResultsViewModel<History_Stream.Moment> Get( Int32 p = 0, Int32 s = 30 )
		{
			using ( var session = this.store.OpenSession() )
			{
				RavenQueryStatistics stats;
				var query = session.Query<History_Stream.Moment, History_Stream>()
					.Statistics( out stats )
					.Skip( p * s )
					.Take( s )
					.OrderBy( m => m.CreatedOn )
					.ProjectFromIndexFieldsInto<History_Stream.Moment>();

				var results = query.ToList();

				var viewModel = new SearchResultsViewModel<History_Stream.Moment>()
				{
					PageIndex = p,
					PageSize = s,
					TotalPages = stats.TotalResults.ToPagesCount( s ),
					TotalResults = stats.TotalResults,
					IsStale = stats.IsStale,
					Results = results
				};

				return viewModel;
			}
		}

		[HttpGet]
		public Boolean PollForEvent( String correlationId = "", String typePattern = "" )
		{
			if ( String.IsNullOrWhiteSpace( correlationId ) && String.IsNullOrWhiteSpace( typePattern ) )
			{
				throw new NotSupportedException( "At least one of the 2 parameters (correlationId or typePattern) must have a value." );
			}

			using ( var session = this.store.OpenSession() )
			{
				var query = session.Query<Events_Stream.Event, Events_Stream>();

				if ( !String.IsNullOrWhiteSpace( correlationId ) )
				{
					query = query.Where( e => e.CorrelationId == correlationId );
				}

				if ( !String.IsNullOrWhiteSpace( typePattern ) )
				{
					query = query.Search( e => e.EventType, typePattern, options: SearchOptions.And, escapeQueryOptions: EscapeQueryOptions.AllowAllWildcards );
				}

				var exists = query.ProjectFromIndexFieldsInto<Events_Stream.Event>().Any();

				return exists;
			}
		}
	}
}
