﻿using System;
using System.Linq;
using System.Net.Http;
using System.Threading;
using System.Threading.Tasks;
using System.Web.Http.Controllers;
using Bootstrapper;
using Topics.Radical.Validation;

namespace WebApiServer.Infrastructure.Filters
{
	public class ApiCorrelationIdActionFilter : System.Web.Http.Filters.IActionFilter
	{
		readonly IOperationContextManager operationContextManager;

		public ApiCorrelationIdActionFilter( IOperationContextManager operationContextManager )
		{
			this.operationContextManager = operationContextManager;
		}

		public Task<HttpResponseMessage> ExecuteActionFilterAsync( HttpActionContext actionContext, CancellationToken cancellationToken, Func<Task<HttpResponseMessage>> continuation )
		{
			Ensure.That( actionContext ).Named( () => actionContext ).IsNotNull();
			Ensure.That( continuation ).Named( () => continuation ).IsNotNull();

			var context = this.operationContextManager.GetCurrent();
			var correlationIdHeader = "x-correlation-id";

			if ( actionContext.Request.Headers.Contains( correlationIdHeader ) )
			{
				var cid = actionContext.Request.Headers.GetValues( correlationIdHeader ).Single();
				context.ForOperation( cid );
			}

			return continuation()
				.ContinueWith( t =>
				{
					if ( !t.IsFaulted && context != null && !String.IsNullOrWhiteSpace( context.CorrelationId ) )
					{
						t.Result.Headers.Add( correlationIdHeader, context.CorrelationId );
					}

					return t;
				} )
				.Unwrap();
		}

		public bool AllowMultiple
		{
			get { return false; }
		}
	}
}